from lume.config.check_list_or_str_item import check_list_or_str_item
from sys import platform


def get_platform():
    if platform == "linux" or platform == "linux2":
        return "linux"
    elif platform == "darwin":
        return "macos"
    elif platform == "win32":
        return "windows"


def concat(l1, l2):
    if l1 is None and l2 is None:
        return None
    elif l1 is None:
        l1 = []
    elif l2 is None:
        l2 = []
    s = l1 + l2
    return s


def check_os_list_or_str_item(kdict, key, required=False, suffix=""):
    kvalue = kdict.get(key)
    if not kvalue and required:
        raise TypeError(f"StepConfig must contains {key}{suffix} variable")

    if isinstance(kvalue, dict):
        platform = get_platform()
        platform_commands = check_list_or_str_item(kvalue, platform)
        all_commands = check_list_or_str_item(kvalue, "all")
        value = concat(platform_commands, all_commands)
    else:
        value = check_list_or_str_item(kdict, key)

    return value



from typing import Dict

from meiga import Result, Error, isSuccess

from lume.src.domain.services.interface_killer_service import IKillerService


class FakeKillerService(IKillerService):
    def info(self) -> Dict:
        return {"name": self.__class__.__name__}

    def execute(self, process: str) -> Result[bool, Error]:
        print("executing")
        return isSuccess



from dataclasses import dataclass
from typing import List, Optional

from lume.config.check_os_list_or_str_item import check_os_list_or_str_item


@dataclass
class InstallConfig:
    run: List[str]
    cwd: Optional[str] = None

    @staticmethod
    def from_dict(kdict):

        run = check_os_list_or_str_item(kdict, "run")

        return InstallConfig(run=run if run else [], cwd=kdict.get("cwd"))



from subprocess import Popen
from typing import Dict

from meiga import Result, Error, Success
from lume.src.domain.services.interface_executor_service import IExecutorService
from lume.src.domain.services.interface_logger import ILogger, INFO


class DetachPopenExecutorService(IExecutorService):
    def __init__(self, logger: ILogger):
        self.logger = logger

    def info(self) -> Dict:
        return {"name": self.__class__.__name__}

    def execute(
        self, command: str, cwd: str, log_filename="setup_detach.log"
    ) -> Result[bool, Error]:

        if not cwd:
            cwd = "."

        log = open(log_filename, "w")
        process = Popen("exec " + command, stdout=log, stderr=log, cwd=cwd, shell=True)

        self.logger.log(INFO, f"Open process (pid={process.pid}) >> {log_filename}")

        return Success(process)



from typing import Dict, List
from lume.config.install_config import InstallConfig
from lume.config.setup_config import SetupConfig
from lume.config.step_config import StepConfig


class Config:
    def __init__(self, yaml_dict: Dict = None):
        if yaml_dict is None:
            self.name = None
            self.settings = {"show_exit_code": False}
            self.install = None
            self.steps = {}
        else:
            self.name = yaml_dict.get("name")
            self.settings = {
                "show_exit_code": yaml_dict.get("settings", {}).get(
                    "show_exit_code", False
                )
            }

            if yaml_dict.get("install"):
                self.install = InstallConfig.from_dict(yaml_dict.get("install"))
            else:
                self.install = InstallConfig(run=[])

            self.steps = {}
            for step_name, step in yaml_dict["steps"].items():
                if step_name == "setup":
                    self.steps[step_name] = SetupConfig.from_dict(step)
                else:
                    self.steps[step_name] = StepConfig.from_dict(step)

    def get_steps(self) -> List[str]:
        return list(self.steps.keys())

    def get_commands(self) -> List[str]:
        commands = []
        commands += self.get_steps()
        if self.install:
            commands.append("install")
        return commands



from dataclasses import dataclass
from dataclasses_json import dataclass_json
from typing import Dict, Optional

from lume.config.dependency_config import DependencyConfig


@dataclass_json
@dataclass
class SetupConfig:
    deps: Dict[str, DependencyConfig]
    output: Optional[str] = "deps"



from typing import Dict

from meiga import Success

from lume.config import SetupConfig
from lume.src.domain.services.interface_logger import ILogger, INFO, WARNING
from lume.src.domain.services.interface_setup_service import ISetupService


class FakeSetupService(ISetupService):
    def __init__(self, setup_config: SetupConfig, logger: ILogger):
        self.setup_config = setup_config
        self.logger = logger

    def info(self) -> Dict:
        return {"name": self.__class__.__name__}

    def execute(self):

        if not self.setup_config:
            self.logger.log(WARNING, f"Empty config for setup")
            return Success()

        self.logger.log(INFO, f"setup: output: {self.setup_config.output}")

        for key, value in self.setup_config.deps.items():
            self.logger.log(INFO, f"{key} : {value}")
        return Success()
